using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;

namespace CommonActivity
{
    [Designer(typeof(SendEmailActivityDesigner), typeof(IDesigner))]
    [ActivityValidatorAttribute(typeof(SendEmailActivityValidator))]
	public partial class SendEmailActivity: Activity
	{
		public SendEmailActivity()
		{
			InitializeComponent();
		}

        public static DependencyProperty FromProperty = DependencyProperty.Register("From", typeof(string), typeof(SendEmailActivity), new PropertyMetadata(null));
        public static DependencyProperty ToProperty = DependencyProperty.Register("To", typeof(string), typeof(SendEmailActivity), new PropertyMetadata(null));

        public static DependencyProperty SendingEmailEvent = DependencyProperty.Register("SendingEmail", typeof(EventHandler), typeof(SendEmailActivity));
        public static DependencyProperty SentEmailEvent = DependencyProperty.Register("SentEmail", typeof(EventHandler), typeof(SendEmailActivity));

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [DescriptionAttribute("The from email that you want to set.")]
        public string From
        {
            get
            {
                return ((string)(base.GetValue(SendEmailActivity.FromProperty)));
            }
            set
            {
                base.SetValue(SendEmailActivity.FromProperty, value);
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [DescriptionAttribute("The to email that you want to set.")]
        public string To
        {
            get
            {
                return ((string)(base.GetValue(SendEmailActivity.ToProperty)));
            }
            set
            {
                base.SetValue(SendEmailActivity.ToProperty, value);
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [DescriptionAttribute("Fired before an email is sent.")]
        public event EventHandler SendingEmail
        {
            add
            {
                base.AddHandler(SendEmailActivity.SendingEmailEvent, value);
            }
            remove
            {
                base.RemoveHandler(SendEmailActivity.SendingEmailEvent, value);
            }
        }


        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [DescriptionAttribute("Fires after an email is sent.")]
        public event EventHandler SentEmail
        {
            add
            {
                base.AddHandler(SendEmailActivity.SentEmailEvent, value);
            }
            remove
            {
                base.RemoveHandler(SendEmailActivity.SentEmailEvent, value);
            }
        }


        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            try
            {
                base.RaiseEvent(SendEmailActivity.SendingEmailEvent, this, EventArgs.Empty);
                this.SendEmail();
                base.RaiseEvent(SendEmailActivity.SentEmailEvent, this, EventArgs.Empty);
                return ActivityExecutionStatus.Closed;
            }
            catch
            {
                return ActivityExecutionStatus.Faulting;
            }
        }

        private void SendEmail()
        {
            Console.WriteLine("Hey,  I am sending an email like it or not to "+ this.To + " and it is coming from " + this.From + "!");
        }
	}


    internal sealed class SendEmailActivityDesignerTheme : ActivityDesignerTheme
    {
        public SendEmailActivityDesignerTheme(WorkflowTheme theme)
            :base (theme)
        {
            this.BorderColor = Color.Black;
            this.BackColorStart = Color.Blue;
            this.BackColorEnd = Color.CornflowerBlue;
            this.BorderStyle = System.Drawing.Drawing2D.DashStyle.Solid;
            this.ForeColor = Color.White;
            this.BackgroundStyle = System.Drawing.Drawing2D.LinearGradientMode.Horizontal;
        }
    }

    [ActivityDesignerThemeAttribute(typeof(SendEmailActivityDesignerTheme))]
    internal sealed class SendEmailActivityDesigner : ActivityDesigner
    {
        protected override void Initialize(Activity activity)
        {
            base.Initialize(activity);
            Bitmap bmp = Resources.eps_open_largelHS;
            bmp.MakeTransparent();
            this.Image = bmp;
            this.Text = "Send Email";
        }
    }

    internal sealed class SendEmailActivityValidator : ActivityValidator
    {
        public override ValidationErrorCollection Validate(ValidationManager manager, object obj)
        {
            ValidationErrorCollection errors = new ValidationErrorCollection();
            SendEmailActivity activity = obj as SendEmailActivity;
            if (activity != null)
            {
                this.ValidateProps(errors, activity);
            }
            return errors;
        }

        private void ValidateProps(ValidationErrorCollection errors, SendEmailActivity activity)
        {
            if (activity.Parent != null)
            {
                if (String.IsNullOrEmpty(activity.From))
                {
                    errors.Add(ValidationError.GetNotSetValidationError(SendEmailActivity.FromProperty.Name));
                }
                if (String.IsNullOrEmpty(activity.To))
                {
                    errors.Add(ValidationError.GetNotSetValidationError(SendEmailActivity.ToProperty.Name));
                }
            }
        }
    }
}
